@@ -23,7 +23,7 @@ from bs4 import BeautifulSoup
 from selenium import webdriver
 from selenium.webdriver import DesiredCapabilities
 
-conf_path = os.path.join(os.getcwd(), 'config.ini')
+conf_path = os.path.join('E:\project\python3-object\spiderman_config_file_package\hgrab\hgrab', 'config.ini')
 
 
 def load_config(file):
@@ -76,6 +76,24 @@ class MysqlDB:
             self.insert(sql, times)
         return True
 
+    # 插入
+    async def insert_async(self, sql, times=0, print_text='insert'):
+        try:
+            await self.cursor.execute(sql)
+            await self.connet.commit()
+        except:
+            times += 1
+            if self.debug:
+                traceback.print_exc()
+            print('failed {}!!! :{}'.format(print_text, sql))
+            if times > 4:
+                traceback.print_exc()
+                return False
+            sleep(1.5)
+            self.reload_database()
+            await self.insert_async(sql, times)
+        return True
+
     # 查询
     def select(self, sql, times=0):
         try:
@@ -95,7 +113,7 @@ class MysqlDB:
         return result
 
     # 存在
-    def has(self, sql, times):
+    def has(self, sql, times=0):
         try:
             self.cursor.execute(sql)
             if self.cursor.fetchone() is None:
@@ -113,6 +131,25 @@ class MysqlDB:
             self.has(sql, times)
         return True
 
+    # 存在
+    async def has_async(self, sql, times=0):
+        try:
+            await self.cursor.execute(sql)
+            if self.cursor.fetchone() is None:
+                return False
+        except:
+            times += 1
+            if self.debug:
+                traceback.print_exc()
+            print('failed has!!! :', sql)
+            if times > 4:
+                traceback.print_exc()
+                return ''
+            sleep(1.5)
+            self.reload_database()
+            await self.has_async(sql, times)
+        return True
+
     # 更新
     def update(self, sql, times=0):
         self.insert(sql, times=times, print_text='update')
@@ -139,7 +176,7 @@ class Downloader:
         else:
             self.config_path = conf_path
         self.conf = load_config(self.config_path)
-        self.headers = json.loads(self.conf.get('headers', 'headers'))
+        self.headers = json.loads(self.conf.get('headers', 'headers_'))
         self.proxyMeta = "http://%(host)s:%(port)s" % {
             "host": self.conf.get(section, 'proxy_host'),
             "port": self.conf.get(section, 'proxy_port'),
@@ -240,6 +277,53 @@ class Downloader:
             html = response.content.decode(encoding)
         return status, html, redirected_url
 
+    # aiohttp异步
+    async def fetch(self, session, url, headers=None, timeout=9, binary=False, debug=None):
+        if not headers:
+            _headers = self.headers
+        if headers:
+            _headers = headers
+        try:
+            async with session.get(url, headers=_headers, timeout=timeout) as response:
+                status = response.status
+                html = await response.read()
+                if not binary:
+                    encoding = cchardet.detect(html)['encoding']
+                    html = html.decode(encoding, errors='ignore')
+                redirected_url = str(response.url)
+        except Exception as e:
+            if debug:
+                traceback.print_exc()
+            msg = 'Failed download: {} | exception: {}, {}'.format(url, str(type(e)), str(e))
+            print(msg)
+            html = ''
+            status = 0
+            redirected_url = url
+        return status, html, redirected_url
+
+    async def post_fetch(self, session, url, data, headers=None, timeout=10, binary=False, debug=None):
+        if not headers:
+            _headers = self.headers
+        if headers:
+            _headers = headers
+        try:
+            async with session.post(url, headers=_headers, data=data, timeout=timeout) as response:
+                status = response.status
+                html = await response.read()
+                if not binary:
+                    encoding = cchardet.detect(html)['encoding']
+                    html = html.decode(encoding, errors='ignore')
+                redirected_url = str(response.url)
+        except Exception as e:
+            if debug:
+                traceback.print_exc()
+            msg = 'Failed download: {} | exception: {}, {}'.format(url, str(type(e)), str(e))
+            print(msg)
+            html = ''
+            status = 0
+            redirected_url = url
+        return status, html, redirected_url
+
     # webdriver+selenium 模拟浏览器
     def get_selenium_driver(self, executable_path=None, wap_use=False):
         option = webdriver.ChromeOptions()
